@@ -3,7 +3,7 @@
  *               similar to SRVANY or Invoker but straight implemented
  *		 to support Cygwin applications.
  *
- * Copyright 2001-2014  Corinna Vinschen, <corinna@vinschen.de>
+ * Copyright 2001-2015  Corinna Vinschen, <corinna@vinschen.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,7 +51,7 @@
 #include "utils.h"
 #include "waitanypid.h"
 
-static const char *SCCSid = "@(#)cygrunsrv V1.61, " __DATE__ "\n";
+static const char *SCCSid = "@(#)cygrunsrv V1.62, " __DATE__ "\n";
 
 struct option longopts[] = {
   { "install", required_argument, NULL, 'I' },
@@ -1978,8 +1978,7 @@ service_main (DWORD argc, LPSTR *argv)
 	  /* Do not report that the service is stopped so that if
 	     recovery options are set, Windows will automatically
 	     restart the service. */
-	  service_main_exitval = s;
-	  _exit (service_main_exitval);
+	  _exit (s);
 	}
       else
 	{
@@ -1998,8 +1997,17 @@ service_main (DWORD argc, LPSTR *argv)
 	syslog (LOG_INFO, "`%s' service stopped, signal %d received",
 		svcname, WTERMSIG (status));
       else
-	syslog (LOG_ERR, "service `%s' failed: signal %d raised",
-		svcname, WTERMSIG (status));
+	{
+	  syslog (LOG_ERR, "service `%s' failed: signal %d raised",
+		  svcname, WTERMSIG (status));
+	  if (neverexits && !shutting_down)
+	    {
+	      /* Do not report that the service is stopped so that if
+		 recovery options are set, Windows will automatically
+		 restart the service. */
+	      _exit (WTERMSIG (status));
+	    }
+	}
       service_main_exitval = 0;
       set_service_status (SERVICE_STOPPED);
     }
